% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% This function constructs a dataset from randomly selected salient   %
% subregions of randomly selected training images, extracts their     %
% deep features, and learns the clustering vectors by kmeans          %
%                                                                     %
%                                                                     %
% selectedTrainingImageFilenames : Filenames of randomly selected     %
%                     training images                                 %
% minCircleRadius   : The size of the smallest circular object        %
%                     (tissue component) to be located                %
% winSize           : The size of a salient subregion                 %
% K                 : The number of clusters                          %
%                                                                     %
% weightMatFile     : The file storing the weights of RBMs. The bias  %
%                     vector names should be B1, B2, ... and          %
%                     the weight matrix names should be W1, W2, ...   %
%                     if you want to change the number of RBMs,       %
%                     you should modify the code given in Line 50 and %
%                     Lines 80-84                                     %
%                                                                     %
%                                                                     %
% V                 : The clustering vectors                          %
%                                                                     %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
function V = learnClusteringVectors(selectedTrainingImageFilenames, weightMatFilename, minCircleRadius, winSize, K)

numberOfRandomlySelectedSubregions = 200;
clusterData = formDataForClustering(selectedTrainingImageFilenames, weightMatFilename, minCircleRadius, winSize, numberOfRandomlySelectedSubregions);
[~, V] = kmeansClustering(clusterData, K);

end
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
function clusterData = formDataForClustering(selectedTrainingImageFilenames, weightMatFilename, minCircleRadius, winSize, selNo)

eval(['load ' weightMatFilename]);

trSize = size(selectedTrainingImageFilenames, 2);
firstIm = imread(selectedTrainingImageFilenames{1});
[dimx, dimy, ~] = size(firstIm);
halfSize    = (winSize - 1) / 2;
subLength   = (halfSize * 2 + 1) * (halfSize * 2 + 1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If you want to use a different number of RBMs (hidden layers), %
% use the name of the last layer's bias vector instead of B4 .   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dimNo       = size(B4, 2);

allCnt = 0;
for i = 1 : trSize

    im = imread(selectedTrainingImageFilenames{i});
    npix = findNucleusPixels(im);
    
    % im_circles is the file that temporarily stores the circles of the
    % current image. If you want to  store the circle information, use a
    % separate name for each image file. This will also fasten your program
    % if you want to run the algorithm with different parameters
    locateTissueCircles(im, 'H&E', 1, minCircleRadius, minCircleRadius, -1, 'im_circles', 0);
    orgCircles = load('im_circles');
    
    cno = size(orgCircles, 1);
    tmpData = zeros(cno, dimNo);
    cnt = 1;
    cperm = randperm(cno);
    for j = 1 : cno
        cx = orgCircles(cperm(j),1);
        cy = orgCircles(cperm(j),2);
        
        if (cx - halfSize >= 1) && (cx + halfSize <= dimx) && (cy - halfSize >= 1) && (cy + halfSize <= dimy)
            subIm = reshape(npix(cx-halfSize : cx+halfSize, cy-halfSize : cy+halfSize),[1 subLength]);

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % You need to change these statements if you want to use %
            % a different number of RBMs (hidden layers)             %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            posP1 = 1 ./ (1 + exp(-(subIm * W1 + B1)));
            posP2 = 1 ./ (1 + exp(-(posP1 * W2 + B2)));
            posP3 = 1 ./ (1 + exp(-(posP2 * W3 + B3)));
            posP4 = 1 ./ (1 + exp(-(posP3 * W4 + B4)));
            tmpData(cnt,:) = posP4;
            
            cnt = cnt + 1;
        end
        
        if(cnt > selNo)
            break
        end
    end

    clusterData(allCnt + 1: allCnt + cnt - 1,:) = tmpData(1:cnt - 1, :);
    allCnt = allCnt + cnt - 1;
end

end
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%


